class Panel < ActiveRecord::Base
  belongs_to :comic
  belongs_to :author
  has_many :panel_pictures, :dependent => :destroy
  has_many :balloons, :dependent => :destroy
  accepts_nested_attributes_for :panel_pictures, :allow_destroy => true
  accepts_nested_attributes_for :balloons, :allow_destroy => true

  def self.max_t comic_id
    Panel.maximum(:t, :conditions => ['comic_id = ?', comic_id]).to_i
  end
  
  def self.find_t comic_id, t
    Panel.find(:first, :conditions => ['comic_id = ? and t = ?', comic_id, t])
  end

  #更新する時にPanelIDをチェックしとかないと勝手に所属先を変えられるゾ！？

  def vdt_save
    f = nil
    max_t = Panel.max_t self.comic_id
    f = Panel.find_t(self.comic_id, self.t) if self.t
    if f
      Panel.update_all('t = t + 1', ['comic_id = ? and t >= ?', self.comic_id, self.t])
    else
      self.t = max_t + 1
    end
    self.save
  end
  
  def move_to new_t
    return true if self.t == new_t
    if self.t > new_t
      Panel.update_all('t = t + 1', ['comic_id = ? and (t >= ? and t < ?)', self.comic_id, new_t, self.t])
    else
      nf = Panel.find_t(self.comic_id, new_t)
      max_t = Panel.max_t self.comic_id
      new_t = max_t if new_t > max_t
      Panel.update_all('t = t - 1', ['comic_id = ? and (t > ? and t <= ?)', self.comic_id, self.t, new_t])
    end
    self.t = new_t
    self.save
  end
  
  def destroy_and_shorten
    Panel.update_all('t = t - 1', ['comic_id = ? and (t > ?)', self.comic_id, self.t])
    self.destroy
  end
  
  def own? author
    return false unless author
    self.author_id == author.id
  end
  
  def sort_by_time
    pe = []
    self.panel_pictures.each do |picture|
      pe[picture.t] = picture
    end
    self.balloons.each do |balloon|
      pe[balloon.t] = balloon
    end
    pe
  end
  
  def each_element
    self.sort_by_time.each do |e|
      yield e
    end
  end
  
  def panel_elements
    res = []
    self.each_element do |elm|
      if elm.kind_of?(PanelPicture)
        res[elm.t] = elm.to_json({:include => :resource_picture})
      end
      if elm.kind_of?(Balloon)
        res[elm.t] = elm.to_json({:include => :speaches})
      end
    end
    res
  end
  
  def to_json_play
    self.to_json :methods => :panel_elements
  end
end
