# -*- encoding: utf-8 -*-
#コマ絵
require 'spec_helper'

describe PanelPicturesController do
  before do
    Factory :admin
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
  end

  describe '一覧表示に於いて' do
    before do
      @pp = Factory :panel_picture, :author_id => @author.id
      sign_in @user
      PanelPicture.stub(:list).and_return([@pp, @pp, @pp])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq PanelPicture.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq PanelPicture.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq PanelPicture.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it 'コマ絵モデルに一覧を問い合わせている' do
        PanelPicture.should_receive(:list).exactly(1)
        get :index
      end
      it '@panel_picturesにリストを取得している' do
        get :index
        assigns(:panel_pictures).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマ絵っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("link").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '作家が絵師でないとき' do
      before do
        Author.any_instance.stub(:artist?).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it '絵師登録ページへ遷移する' do
          get :index
          response.should redirect_to new_artist_path
        end
      end
      context 'json形式' do
        it '例外403 forbiddenを返す' do
          lambda{
            get :index, :format => :json
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
    end
  end
  
  describe 'コマ絵数取得に於いて' do
    before do
      PanelPicture.should_receive(:visible_count).and_return(3)
#      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがHash構造になっていてコマ絵数が3である' do
          get :count, :format => :json
          json = JSON.parse response.body
          json["count"].should == 3
        end
      end
    end
  end

end
